/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.samples.livetour.model.dataaccess;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import org.ow2.opensuit.samples.livetour.model.ExactStringTokenizer;
import org.ow2.opensuit.samples.livetour.model.to.BookTO;


/**
 * This class gives acces to the movies database.
 */
public class BestSellersDataBase
{
	private static final String CHARSET = "utf-16";
	private static final char SEPAR = '\t';
	private static final BookTO[] BOOK_ARRAY = {};
	private static BestSellersDataBase database;
	
	/**
	 * Returns the database connection object.
	 * @return
	 * @throws DataBaseError In case of a technical error.
	 */
	public static BestSellersDataBase getDataBase() throws DataBaseError
	{
		if(database == null)
		{
			URL url = BestSellersDataBase.class.getClassLoader().getResource("best_sellers.txt");
			File dbDir = new File(url.getFile()).getParentFile();
			database = new BestSellersDataBase(dbDir);
			database.load();
		}
		return database;
	}
	
	
	private File databaseDir;
	private ArrayList<BookTO> books;
	private ArrayList<String> languages;
	
	private final DateFormat DATE_FORMAT = new SimpleDateFormat("yyyy");

	private BestSellersDataBase(File iDatabaseDir)
	{
		databaseDir = iDatabaseDir;
//		load();
	}
	private void load()
	{
		try
		{
			loadBooks();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	private void loadBooks() throws IOException
	{
		// --- 1: load directors
		books = new ArrayList<BookTO>();
		languages = new ArrayList<String>();
		File file = new File(databaseDir, "best_sellers.txt");
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(file), CHARSET));
		try
		{
			String curline = null;
			int id = 0;
			while((curline = reader.readLine()) != null)
			{
//				curline = curline.trim();
				if(curline.length() == 0 || curline.startsWith("#")){
					// skip
					continue;
				}
				
//            	System.out.println("line "+id+": "+curline);
				
				// --- create a new book
//				String[] values = curline.split("\t");
				String[] values = ExactStringTokenizer.split(curline, SEPAR);
				BookTO book = new BookTO();
				book.setId(id++);
				book.setTitle(values[0]);
				book.setAuthor(values[1]);
				String[] langs = values[2].split(",");
				book.setLanguages(langs);
				for(int i=0; i<langs.length; i++)
				{
					if(!languages.contains(langs[i])){
						languages.add(langs[i]);
					}
				}
				try
                {
					book.setEditionDate(DATE_FORMAT.parse(values[3]));
                }
                catch(ParseException e)
                {
                	System.out.println("line "+id+": could not parse date ("+values[3]+")");
                }
				try
                {
				book.setCopies(Float.parseFloat(values[4]));
                }
                catch(NumberFormatException e)
                {
                	System.out.println("line "+id+": could not parse number ("+values[4]+")");
                }
				
				books.add(book);
			}
		}
		finally
		{
			reader.close();
		}
	}
	/*
	private void saveBooks()
	{
		File file = new File(databaseDir, "best_sellers.txt");
		PrintWriter writer = null;
        try
        {
	        writer = new PrintWriter(file, CHARSET);
        }
        catch(FileNotFoundException e)
        {
	        e.printStackTrace();
	        return;
        }
		catch (UnsupportedEncodingException e)
		{
			e.printStackTrace();
	        return;
		}
		
		writer.println("# Books");
		for(int i=0; i<books.size(); i++)
		{
			BookTO book = books.get(i);
//			writer.print(String.valueOf(book.getId()));
//			writer.print(SEPAR);
			writer.print(book.getTitle());
			writer.print(SEPAR);
			writer.print(book.getAuthor());
			writer.print(SEPAR);
			writer.print(book.getLanguages());
			writer.print(SEPAR);
			writer.print(DATE_FORMAT.format(book.getEditionDate()));
			writer.print(SEPAR);
			writer.print(String.valueOf(book.getCopies()));
			writer.println();
		}
		
		writer.close();
	}
	*/

	// ========================================================================================
	// === Books Methods
	// ========================================================================================
	private int getBookIndex(int iId)
	{
		for(int i=0; i<books.size(); i++)
		{
			if(books.get(i).getId() == iId){
				return i;
			}
		}
		return -1;
	}
	/**
	 * Returns the movie description from its ID
	 * @param iId
	 * @return
	 * @throws DataBaseError This method may either throw a TECHNICAL_ERROR in case of a 
	 * technical error (communication failure, ...), or DATA_NOT_FOUND_ERROR if no
	 * movie was found in the database with the specified ID.
	 */
	public BookTO Book(int iId) throws DataBaseError
	{
		int idx = getBookIndex(iId);
		
		if(idx < 0){
			throw new DataBaseError(DataBaseError.DATA_NOT_FOUND_ERROR, "Book "+iId+" not found.");
		}
		
		try {
            return books.get(idx).clone();
        } catch (final CloneNotSupportedException e) {
            throw new DataBaseError(DataBaseError.TECHNICAL_ERROR, e.getMessage());
        }
	}
	/**
	 * Counts all books
	 * @return
	 */
	public int countAllBooks() throws DataBaseError
	{
		return books.size();
	}
	private BookTO[] applyRequestOptions(ArrayList<BookTO> list, RequestOptions iOptions)
	{
		BookTO[] ret = list.toArray(BOOK_ARRAY);
		
		if(iOptions != null)
		{
			// --- 1: sort
			if(iOptions.hasSortOption()){
				Arrays.sort(ret, new BooksComparator(iOptions.getSortCriteria(), iOptions.isSortAscendingly()));
			}
			
			// --- 2: paginate
			if(iOptions.hasPagination())
			{
				BookTO[] paginated = new BookTO[Math.min(iOptions.getNbOfResults(), ret.length-iOptions.getFirstResult())];
				System.arraycopy(ret, iOptions.getFirstResult(), paginated, 0, paginated.length);
				ret = paginated;
			}
		}
		
		return ret;
	}
	/**
	 * Returns all movies
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	public BookTO[] getAllBooks(RequestOptions iOptions) throws DataBaseError
	{
		return applyRequestOptions(books, iOptions);
	}
	public ArrayList<String> getAllLanguages()
	{
		return languages;
	}
	/*
	private int createMovieId()
	{
		Random rd = new Random();
		while(true)
		{
//			String id = String.valueOf(Math.abs(rd.nextInt()));
			int id = Math.abs(rd.nextInt());
            if(getMovieIndex(id) < 0)
            	return id;
		}
	}
	*/
	/**
	 * Sets the given movie.<br>
	 * This method either overwrites an existing movie or creates a new movie in the database.
	 * @param iMovie
	 */
	/*
	public void setMovie(MovieTO iMovie) throws DataBaseError
	{
		int idx = iMovie.getId() == 0 ? -1 : getMovieIndex(iMovie.getId());
		if(idx < 0)
		{
			// --- add a new movie
			iMovie.setId(createMovieId());
			iMovie = iMovie.clone();
			movies.add(iMovie);
		}
		else
		{
			// --- update an existing movie
			iMovie = iMovie.clone();
			movies.set(idx, iMovie);
		}
		saveMovies();
	}
	*/
	/**
	 * Removes the given movie from the database.
	 * @param iId
	 */
	/*
	public void removeMovie(int iId) throws DataBaseError
	{
		int idx = getMovieIndex(iId);
		if(idx < 0)
			return;
		movies.remove(idx);
		saveMovies();
	}
	*/
	/**
	 * Searches movies with title
	 * @param title
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	public BookTO[] searchBooksByTitle(String title, RequestOptions iOptions) throws DataBaseError
	{
		// --- 1: search
		title = title.toLowerCase();
		
		ArrayList<BookTO> results = new ArrayList<BookTO>();
		for(int i=0; i<books.size(); i++)
		{
			BookTO book = books.get(i);
			if(book.getTitle().toLowerCase().contains(title))
			{
				results.add(book);
			}
		}
		
		// --- 2: process options
		return applyRequestOptions(results, iOptions);
	}
	public int countBooksByTitle(String title) throws DataBaseError
	{
		int count = 0;
		for(int i=0; i<books.size(); i++)
		{
			BookTO book = books.get(i);
			if(book.getTitle().toLowerCase().contains(title))
			{
				count++;
			}
		}
		return count;
	}
	/**
	 * Searches movies by director name
	 * @param author
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	/*
	public BookTO[] searchBooksByAuthor(String author, RequestOptions iOptions) throws DataBaseError
	{
		ArrayList<BookTO> results = new ArrayList<BookTO>();
		if(matchingDirectors != null && matchingDirectors.length > 0)
		{
			for(int i=0; i<matchingDirectors.length; i++)
			{
				MovieTO[] movies = getMoviesFromDirector(matchingDirectors[i].getId(), null);
				for(int j=0; j<movies.length; j++)
					results.add(movies[j]);
			}
		}
		
		// --- 2: process options
		return applyRequestOptions(results, iOptions);
	}
	
	public int countMoviesByDirector(String director) throws DataBaseError
	{
		PersonTO[] matchingDirectors = searchDirectorByName(director, null);
		
		int count = 0;
		if(matchingDirectors != null && matchingDirectors.length > 0)
		{
			for(int i=0; i<matchingDirectors.length; i++)
			{
				MovieTO[] movies = getMoviesFromDirector(matchingDirectors[i].getId(), null);
				count += movies.length;
			}
		}
		return count;
	}
	*/

	// ==============================================================================
	// === Books Comparator (to implement sorting)
	// ==============================================================================
	/**
	 * Sort criteria: sort by ID
	 */
	public static final int SORT_BY_ID = 1;
	/**
	 * Sort criteria: sort by title
	 */
	public static final int SORT_BY_TITLE = 2;
	/**
	 * Sort criteria: sort by author
	 */
	public static final int SORT_BY_AUTHOR = 4;
	/**
	 * Sort criteria: sort by print date
	 */
	public static final int SORT_BY_RELEASE_DATE = 5;
	/**
	 * Sort criteria: sort by print date
	 */
	public static final int SORT_BY_NUMBER_OF_COPIES = 3;
	
	private static class BooksComparator implements Comparator<BookTO>
	{
		private int crit;
		private int sign;
		
		public BooksComparator(int iSortingCriteria, boolean iSortAscendingly)
		{
			crit = iSortingCriteria;
			sign = iSortAscendingly ? 1 : -1;
		}
		public int compare(BookTO m1, BookTO m2)
		{
			switch(crit)
			{
				case SORT_BY_ID:
				    
					return sign*(m1.getId()-m2.getId());
					
				case SORT_BY_TITLE:
				    
					if(m1.getTitle() == null)
					{
						if(m2.getTitle() == null){
							return 0;
						}
						
						return -sign;
					}
					
					if(m2.getTitle() == null){
						return sign;
					}
					
					return sign * m1.getTitle().compareTo(m2.getTitle());
					
				case SORT_BY_AUTHOR:
				    
				if(m1.getAuthor() == null)
					{
						if(m2.getAuthor() == null){
							return 0;
						}
						
						return -sign;
					}
				
					if(m2.getAuthor() == null){
						return sign;
					}
					
					return sign * m1.getAuthor().compareTo(m2.getAuthor());
					
				case SORT_BY_RELEASE_DATE:
				    
					if(m1.getEditionDate() == null)
					{
						if(m2.getEditionDate() == null){
							return 0;
						}
						
						return -sign;
					}
					
					if(m2.getEditionDate() == null){
						return sign;
					}
					
					return sign * m1.getEditionDate().compareTo(m2.getEditionDate());
					
				default: // SORT_BY_NUMBER_OF_COPIES:
				    
					return sign * (int)(m2.getCopies() - m1.getCopies());
			}
		}
	}
	public static void main(String[] args) throws DataBaseError
	{
		BestSellersDataBase db = BestSellersDataBase.getDataBase();
		BookTO[] books = db.getAllBooks(null);
		for(int i=0; i<books.length; i++)
		{
			System.out.println(" -"+i+": "+books[i]);
		}
	}
}
