/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.ow2.opensuit.samples.livetour.test;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.ow2.opensuit.core.impl.file.fake_j2ee.FakeRequest;
import org.ow2.opensuit.core.impl.file.fake_j2ee.FakeResponse;
import org.ow2.opensuit.core.impl.file.fake_j2ee.FakeServletContext;
import org.ow2.opensuit.core.impl.file.fake_j2ee.FakeSession;
import org.ow2.opensuit.core.impl.tools.Validator.J2EEValidator;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.interfaces.IOpenSuitInstContext;
import org.ow2.opensuit.xml.interfaces.IRequestProcessor;
import org.ow2.opensuit.xmlmap.XmlInstantiator;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.w3c.tidy.Tidy;

/**
 * This test case implements some Open SUIT integration tests:
 *  - instantiate Livetour XML model
 *  - process some requests
 *  
 * @author Adrien Ruffie / Pierre Smeyers
 */
public class LiveTourTestCase
{
	private File outputHtmlDir;
    /**
     * Allow to recover arguments for main() invocation of <XsdGenerator>, from
     * test.properties located into src/test/resources/test.properties
     * 
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception {
    	outputHtmlDir = new File("target/test/livetour");
    	outputHtmlDir.mkdirs();
    }
    
    /**
     * Allow to test the xsd generation from opensuit core data structure
     * @throws Exception 
     */
    @Test
    public void testMain() throws Exception
    {
    	// --- 1: instantiate Open SUIT model
    	File webRootDir = new File("src/main/webapp");
    	File xmlFile = new File("src/main/webapp/WEB-INF/DemoApp.xml");
    	
		IOpenSuitInstContext validSupport = new J2EEValidator(webRootDir);
		XmlInstantiator instantiator = XmlInstantiator.instantiate(xmlFile.toURI().toURL(), validSupport);
		
		// --- 2: check there is no error
		instantiator.dumpMessages(IInitializationSupport.WARNING, new PrintWriter(System.out));
		Assert.assertFalse("XML model has errors", instantiator.hasMessages(IInitializationSupport.ERROR));
		
		// --- 3: check top object implements IRequestProcessor
		Assert.assertTrue("Root instantiated object does not implement IRequestProcessor.", instantiator.getRoot() instanceof IRequestProcessor);
		IRequestProcessor requestProcessor = (IRequestProcessor)instantiator.getRoot();
		
		// --- run some requests
		FakeServletContext srvCtx = new FakeServletContext();
		FakeSession session = new FakeSession(srvCtx);
		FakeResponse response;
		
		// --- Welcome
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Welcome", "Welcome.html");
		
		// --- Overview
		// ------------
		// menu rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=menu", "OverviewMenu.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());
		
		// basic table rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=table", "OverviewTable.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// view form rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=forms&tab_forms=view", "OverviewViewForm.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// edit form rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=forms&tab_forms=edit", "OverviewEditForm.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// viewedit form rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=forms&tab_forms=viewedit", "OverviewViewEditForm.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// Inform box rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=popups&tab_forms=viewedit&_hdl_=/pages/Overview/Info_/invoke", "OverviewInform.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		
		// AskBox rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=popups&tab_forms=viewedit&_hdl_=/pages/Overview/Ask_/invoke", "OverviewAsk.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		
		// tabs rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Overview?tab_main=tabs", "OverviewTabs.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		
		// Table
		// -----
		// sort by title
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Tables?tab_main=sorting&table.sort=1", "TablesSortByString.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// sort by date
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Tables?tab_main=sorting&table.sort=-4", "TablesSortByDate.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// filter text
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Tables?tab_main=filtering&table.filter.1=mao", "TablesTextFilter.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// filter enum
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Tables?tab_main=filtering&table.filter.2=German", "TablesComboFilter.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// toolbar rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Tables?tab_main=toolbar", "TablesToolbar.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		
		// Form
		// ----
		// all fields rendering
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Forms?tab_main=fields", "FormsFields.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// sections and additional buttons
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Forms?tab_main=section", "FormsAddButtons.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// dynamic editability
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/Forms?tab_main=editable", "FormsEditable.html");
		Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatusCode());

		// --- test errors
		// page not found error
		response = processRequest(requestProcessor, session, "/livetour/GUI/pg/pages/NoSuchPage", "error1.html");
		Assert.assertEquals(HttpServletResponse.SC_NOT_FOUND, response.getStatusCode());
    }
    
    private FakeResponse processRequest(IRequestProcessor requestProcessor, HttpSession session, String url, String outputFileName) throws IOException
    {
		FakeRequest req = new FakeRequest(UrlBuilder.parse(url, null), session);
		FakeResponse resp = new FakeResponse();
		requestProcessor.service(req, resp);
		
		FileWriter output = new FileWriter(new File(outputHtmlDir, outputFileName));
		output.write(resp.getResponseContent().toString());
		output.flush();
		output.close();
		
		// check HTML 4.01 transitional validity
		if(resp.getStatusCode() == HttpServletResponse.SC_OK)
		{
	        Tidy tidy = new Tidy(); 
	        StringWriter errorsWriter = new StringWriter();
	        tidy.setErrout(new PrintWriter(errorsWriter));
	        tidy.parse(new ByteArrayInputStream(resp.getResponseContent().toString().getBytes()), null);
	        if(!errorsWriter.getBuffer().toString().contains("no warnings or errors were found"))
	        {
	            System.err.println("Validation errors for "+url+" (see "+outputFileName+"):");
	            System.err.println(errorsWriter.getBuffer().toString());
	            System.err.println();
//	            Assert.assertTrue("Warning or errors have been found during HTML 4.01 transitional validity check", errorsWriter.getBuffer().toString().contains("no warnings or errors were found"));
	        }
		}
		return resp;
    }
}
